#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Indicator developed  by Edwin Coppock.
    /// </summary>
    [Description("Indicator developed  by Edwin Coppock.")]
    [Gui.Design.DisplayName("Coppock")]
    public class Coppock : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int rOCPeriod1 = 14; // Default setting for ROCPeriod1
            private int rOCPeriod2 = 11; // Default setting for ROCPeriod2
            private int wMAPeriod = 10; // Default setting for WMAPeriod
        	private DataSeries sum = null;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.MidnightBlue), PlotStyle.Line, "Plot0"));
            Add(new Line(Color.FromKnownColor(KnownColor.Black), 0, "ZeroLine"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			sum = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            sum.Set(ROC(Input, ROCPeriod1)[0] + ROC(Input, ROCPeriod2)[0]);
			Plot0.Set(WMA(sum, WMAPeriod)[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int ROCPeriod1
        {
            get { return rOCPeriod1; }
            set { rOCPeriod1 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int ROCPeriod2
        {
            get { return rOCPeriod2; }
            set { rOCPeriod2 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int WMAPeriod
        {
            get { return wMAPeriod; }
            set { wMAPeriod = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Coppock[] cacheCoppock = null;

        private static Coppock checkCoppock = new Coppock();

        /// <summary>
        /// Indicator developed  by Edwin Coppock.
        /// </summary>
        /// <returns></returns>
        public Coppock Coppock(int rOCPeriod1, int rOCPeriod2, int wMAPeriod)
        {
            return Coppock(Input, rOCPeriod1, rOCPeriod2, wMAPeriod);
        }

        /// <summary>
        /// Indicator developed  by Edwin Coppock.
        /// </summary>
        /// <returns></returns>
        public Coppock Coppock(Data.IDataSeries input, int rOCPeriod1, int rOCPeriod2, int wMAPeriod)
        {
            checkCoppock.ROCPeriod1 = rOCPeriod1;
            rOCPeriod1 = checkCoppock.ROCPeriod1;
            checkCoppock.ROCPeriod2 = rOCPeriod2;
            rOCPeriod2 = checkCoppock.ROCPeriod2;
            checkCoppock.WMAPeriod = wMAPeriod;
            wMAPeriod = checkCoppock.WMAPeriod;

            if (cacheCoppock != null)
                for (int idx = 0; idx < cacheCoppock.Length; idx++)
                    if (cacheCoppock[idx].ROCPeriod1 == rOCPeriod1 && cacheCoppock[idx].ROCPeriod2 == rOCPeriod2 && cacheCoppock[idx].WMAPeriod == wMAPeriod && cacheCoppock[idx].EqualsInput(input))
                        return cacheCoppock[idx];

            Coppock indicator = new Coppock();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.ROCPeriod1 = rOCPeriod1;
            indicator.ROCPeriod2 = rOCPeriod2;
            indicator.WMAPeriod = wMAPeriod;

            Coppock[] tmp = new Coppock[cacheCoppock == null ? 1 : cacheCoppock.Length + 1];
            if (cacheCoppock != null)
                cacheCoppock.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCoppock = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Indicator developed  by Edwin Coppock.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Coppock Coppock(int rOCPeriod1, int rOCPeriod2, int wMAPeriod)
        {
            return _indicator.Coppock(Input, rOCPeriod1, rOCPeriod2, wMAPeriod);
        }

        /// <summary>
        /// Indicator developed  by Edwin Coppock.
        /// </summary>
        /// <returns></returns>
        public Indicator.Coppock Coppock(Data.IDataSeries input, int rOCPeriod1, int rOCPeriod2, int wMAPeriod)
        {
            return _indicator.Coppock(input, rOCPeriod1, rOCPeriod2, wMAPeriod);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Indicator developed  by Edwin Coppock.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Coppock Coppock(int rOCPeriod1, int rOCPeriod2, int wMAPeriod)
        {
            return _indicator.Coppock(Input, rOCPeriod1, rOCPeriod2, wMAPeriod);
        }

        /// <summary>
        /// Indicator developed  by Edwin Coppock.
        /// </summary>
        /// <returns></returns>
        public Indicator.Coppock Coppock(Data.IDataSeries input, int rOCPeriod1, int rOCPeriod2, int wMAPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Coppock(input, rOCPeriod1, rOCPeriod2, wMAPeriod);
        }

    }
}
#endregion
